#version 330

uniform sampler2D colorTexture;
uniform sampler2D specularTexture;

uniform sampler2DArrayShadow shadowMaps;

uniform bool hasSpecular;
uniform int shininess;

uniform sampler2D normalTexture;
uniform bool hasNormal;

//uniform vec3 dir_light_col;
uniform float ambient_int;
uniform float dir_light_int;
uniform vec3 dir_light_vec;

uniform mat4 depthMVP[4];
uniform float splitD[4];

in vec2 texCoord;
in vec3 N; //normal
in vec3 V; //view

in vec3 T; //tangent
in vec3 B; //bitangent

in vec3 WorldPos;

out vec3 FragColor;

void main()
{
	vec3 Normal;
	//Normal
	if (hasNormal){
		//Read Normal from Texture
		vec3 tex_Normal = normalize(texture2D( normalTexture, texCoord).rgb*2.0 - 1.0); //in tangent Space

		//Transform to Worldspace
		mat3 TBN = mat3(normalize(T), normalize(B), normalize(N));
		Normal = TBN * tex_Normal;

	} else {
		Normal = N;
	}


	//SHADOW MAPPING
	int index=0;
	// find the appropriate depth map to look up in based on the depth of this fragment
	if(gl_FragCoord.z < splitD[0])
		index = 0;
	else if(gl_FragCoord.z < splitD[1])
		index = 1;
	else if(gl_FragCoord.z < splitD[2])
		index = 2;
	else
		index = 3;
	
	//calculate shadow coords
	vec4 ShadowCoord = depthMVP[index] * vec4(WorldPos,1.0f);
	
	float shadow = (texture(shadowMaps,vec4(ShadowCoord.xy, index, ShadowCoord.z)));
	shadow = ShadowCoord.x < 0 || ShadowCoord.x > 1 || ShadowCoord.y < 0 || ShadowCoord.y > 1 ? 0 : shadow;
	
	
	//Diffuse Part
	vec3 tex_col = vec3(texture2D(colorTexture, texCoord));

	vec3 H = normalize(normalize(V)+normalize(-dir_light_vec));

	float cosNL = max(dot(normalize(Normal), normalize(-dir_light_vec)), 0.0);

	//float factor = cosNL*(1-dir_light_int)+dir_light_int;
	float factor = shadow*cosNL*(dir_light_int)+ambient_int;

	tex_col = vec3(tex_col.r*factor, tex_col.g*factor, tex_col.b*factor); 

	//Specular Part
	vec3 spec_col;
	if (hasSpecular) {
		spec_col = vec3(texture2D(specularTexture, texCoord));
	} else {
		spec_col = vec3(1,1,1);
	}
	spec_col = spec_col * shadow* cosNL* pow(max(dot(normalize(Normal), H), 0),shininess);

	FragColor = tex_col+spec_col;
}