#version 330

uniform float TimeDiff;
uniform vec3 DirectionalForce; //direction
uniform vec4 PointForce[8]; //Position + 4. coord: strength
uniform int PointForceCount; //number of active point forces
uniform float MaxSpeed; // a value to limit the speed of the particles; is already multiplied by TimeDiff
uniform vec4 EndColor; //the color of the particle at the end of its lifetime
uniform float RandomStrength; //the intensity of random influence on moving

uniform unsigned int Seed; //for random function

//particle-create-related uniforms
uniform bool EmitParticles; //if true, particles are created
uniform vec3 RootPos; //upper-left-back-corner-pos of the StartBox
uniform vec3 StartBox; //size of the spawn-area
uniform vec4 StartColor; //the color of the particle at the beginning
uniform float Lifetime; //the lifetime of the particles

layout (triangles) in;
layout (triangle_strip, max_vertices = 3) out; //workaround because single points won't render on my graphic card

in vec3 P_position[];
in vec4 P_color[];
in vec3 P_velocity[];
in float P_lifetime[];

out vec3 OutPosition;
out vec4 OutColor;
out vec3 OutVelocity;
out float OutLifetime;

unsigned int varSeed = Seed;

vec3 randomVec()
{
	vec3 result;
	varSeed = (varSeed * 1103515245u + 12345u); result.x = float(varSeed);
	varSeed = (varSeed * 1103515245u + 12345u); result.y = float(varSeed);
	varSeed = (varSeed * 1103515245u + 12345u); result.z = float(varSeed);
	return TimeDiff * ((result / 4294967296.0) - vec3(0.5,0.5,0.5));
}

vec3 randomStartPos()
{
	vec3 result;
	varSeed = (varSeed * 1103515245u + 12345u); result.x = float(varSeed) / 4294967296.0 * StartBox.x;
	varSeed = (varSeed * 1103515245u + 12345u); result.y = float(varSeed) / 4294967296.0 * StartBox.y;
	varSeed = (varSeed * 1103515245u + 12345u); result.z = float(varSeed) / 4294967296.0 * StartBox.z;
	return result;
}

void main()
{
	varSeed = varSeed + uint(gl_PrimitiveIDIn);
	if (EmitParticles)
	{
		for (int i = 0; i < 3; i++)
		{
			OutPosition = RootPos + randomStartPos();
			OutColor = StartColor;
			OutVelocity = vec3(0.0,0.0,0.0);
			OutLifetime = Lifetime;
			EmitVertex();
		}
		EndPrimitive();
		return;
	}

	//if the first of the 3 particles is dead they are all removed
	if (P_lifetime[0] <= 0)
	{
		return;
	}

	//Update

	for (int i = 0; i < 3; i++)
	{
		varSeed = varSeed + uint(gl_PrimitiveIDIn);
		OutLifetime = P_lifetime[i] - TimeDiff;

		OutPosition = P_position[i] + P_velocity[i] * TimeDiff;

		vec3 force = DirectionalForce + randomVec() * RandomStrength;
		for (int i = 0; i < PointForceCount; i++)
		{
			vec3 forceDir = OutPosition - PointForce[i].xyz;
			force += (PointForce[i].w / max(pow(length(forceDir),2),0.3)) * forceDir;
		}
		OutVelocity = P_velocity[i] + force * TimeDiff;

		float speed = length(OutVelocity);
		if (speed > MaxSpeed)
		{
			OutVelocity *= MaxSpeed / speed;
		}

		float f = TimeDiff / P_lifetime[0];
		OutColor.r = P_color[i].r + (EndColor.r - P_color[i].r) * f;
		OutColor.g = P_color[i].g + (EndColor.g - P_color[i].g) * f;
		OutColor.b = P_color[i].b + (EndColor.b - P_color[i].b) * f;
		OutColor.a = P_color[i].a + (EndColor.a - P_color[i].a) * f;

		//gl_Position = OutParticle.position;
		EmitVertex();
	}
	EndPrimitive();
}