#version 330

uniform sampler2D colorTexture;
uniform sampler2D colorTexture2;

uniform sampler2D specularTexture;
uniform sampler2D specularTexture2;

uniform int shininess;

uniform sampler2D normalTexture;
uniform sampler2D normalTexture2;

uniform sampler2DArrayShadow shadowMaps;

uniform float ambient_int; //ambient light intensity
uniform float dir_light_int; //ambient light strength
uniform vec3 dir_light_vec; //must be normalized

uniform float texture1_scaling;
uniform float texture2_scaling;

uniform mat4 depthMVP[4];
uniform float splitD[4];

uniform int debugCSM = 0; //debug cascaded shadow maps

in vec3 WorldPos;

in vec3 N; //normal
in vec3 V; //view

in vec3 T; //tangent
in vec3 B; //bitangent

//interpolation values for textures
in float factor1;
in float factor2;

in vec3 TexMapPos;

//in vec4 ShadowCoord;

layout(location = 0) out vec3 FragColor;

void main()
{
	vec3 texCoord = TexMapPos * texture1_scaling;
	vec3 tex_Normal1 = normalize(texture2D(normalTexture, texCoord.xz).rgb*2.0 - 1.0); //in tangent Space
	vec3 tex_col1 = vec3(texture2D(colorTexture, texCoord.xz));
	vec3 spec_col1= vec3(texture2D(specularTexture, texCoord.xz));
	texCoord = TexMapPos * texture2_scaling;
	texCoord.y = -texCoord.y;
	vec3 tex_Normal2 = normalize(texture2D(normalTexture2, texCoord.zy).rgb*2.0 - 1.0); //in tangent Space
	vec3 tex_col2 = vec3(texture2D(colorTexture2, texCoord.zy));
	vec3 spec_col2 = vec3(texture2D(specularTexture2, texCoord.zy));
	vec3 tex_Normal3 = normalize(texture2D(normalTexture2, texCoord.xy).rgb*2.0 - 1.0); //in tangent Space
	vec3 tex_col3 = vec3(texture2D(colorTexture2, texCoord.xy));
	vec3 spec_col3 = vec3(texture2D(specularTexture2, texCoord.xy));

	vec3 tex_col;
	vec3 spec_col;

	vec3 Tn = normalize(T);
	vec3 Bn = normalize(B);
	vec3 Nn = normalize(N);
	
	//Transform to Worldspace
	tex_Normal1 = tex_Normal1.x * Bn + tex_Normal1.y * Tn + tex_Normal1.z * Nn;
	if (Bn.y > 0)
		tex_Normal2 = tex_Normal2.x * Tn - tex_Normal2.y * Bn + tex_Normal2.z * Nn; else
		tex_Normal2 = tex_Normal2.x * Tn + tex_Normal2.y * Bn + tex_Normal2.z * Nn;
	if (Tn.y > 0)
		tex_Normal3 = tex_Normal3.x * Bn - tex_Normal3.y * Tn + tex_Normal3.z * Nn; else
		tex_Normal3 = tex_Normal3.x * Bn + tex_Normal3.y * Tn + tex_Normal3.z * Nn;

	vec3 Normal = mix(tex_Normal3, tex_Normal2, factor2);
	Normal = normalize(mix(Normal, tex_Normal1, factor1));

	tex_col = mix(tex_col3, tex_col2, factor2);
	tex_col = mix(tex_col, tex_col1, factor1);

	spec_col = mix(spec_col3, spec_col2, factor2);
	spec_col = mix(spec_col, spec_col1, factor1);

	vec3 H = normalize(normalize(V)-normalize(dir_light_vec));

	float cosNL = max(dot(Normal, -normalize(dir_light_vec)), 0.0);
	
	//SHADOW MAPPING
	/*float bias = 0.002*tan(acos(cosNL)); // cosTheta is dot( n,l ), clamped between 0 and 1
	bias = clamp(bias, 0,0.01);*/
	int index=0;
	// find the appropriate depth map to look up in based on the depth of this fragment
	if(gl_FragCoord.z < splitD[0])
		index = 0;
	else if(gl_FragCoord.z < splitD[1])
		index = 1;
	else if(gl_FragCoord.z < splitD[2])
		index = 2;
	else
		index = 3;
	
	//calculate shadow coords
	vec4 ShadowCoord = depthMVP[index] * vec4(WorldPos,1.0f);
	
	float shadow = (texture(shadowMaps,vec4(ShadowCoord.xy, index, ShadowCoord.z)));
	//float shadow = 1.0;
	//float shadow = texture(shadowMaps,vec3(ShadowCoord.xy, 0)).w;
	shadow = ShadowCoord.x < 0 || ShadowCoord.x > 1 || ShadowCoord.y < 0 || ShadowCoord.y > 1 ? 0 : shadow;
	
	//float factor = cosNL*(1-dir_light_int)+dir_light_int;
	float factor = cosNL*(dir_light_int)*shadow+ambient_int;

	tex_col *= factor;

	//Specular Part
	
	spec_col = spec_col * shadow* cosNL* pow(max(dot(Normal, H), 0),shininess);

	FragColor = tex_col+spec_col;

	//FragColor = vec3(cosNL,cosNL,cosNL);
	//if (factor==0)
	//	FragColor = vec3(1, 0, 0);
	
	//debugCSM
	if (debugCSM > 0) {
		vec4 sc = depthMVP[debugCSM-1] * vec4(WorldPos,1.0f);
		if (sc.x > 0 && sc.x < 1 && sc.y > 0 && sc.y < 1 && sc.z > 0 && sc.z < 1) {
			if (debugCSM != index+1)
				FragColor *= 0.6;

			if (debugCSM == 1)
				FragColor += vec3(0.2,0,0);
			else if (debugCSM == 2)
				FragColor += vec3(0,0.2,0);
			else if (debugCSM == 3)
				FragColor += vec3(0,0,0.2); else
				FragColor += vec3(0.2,0.2,0);
		} else
			FragColor *= 0.2;
	}
}
